"use strict";
// Copyright (c) Jupyter Development Team.
// Distributed under the terms of the Modified BSD License.
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var apputils_1 = require("@jupyterlab/apputils");
var coreutils_1 = require("@jupyterlab/coreutils");
var React = require("react");
/**
 * Prompt the user what do about companion packages, if present
 *
 * @param builder the build manager
 */
function presentCompanions(kernelCompanions, serverCompanion, serviceManager) {
    var entries = [];
    if (serverCompanion) {
        entries.push(React.createElement("p", null,
            "This package has indicated that it needs a corresponding server extension:",
            React.createElement("code", null, serverCompanion.base.name)));
    }
    if (kernelCompanions) {
        entries.push(React.createElement("p", null, "This package has indicated that it needs a corresponding package for the kernel."));
        for (var _i = 0, kernelCompanions_1 = kernelCompanions; _i < kernelCompanions_1.length; _i++) {
            var entry = kernelCompanions_1[_i];
            entries.push(React.createElement("p", null,
                "The package",
                React.createElement("code", null, entry.kernelInfo.base.name),
                ", is required by the following kernels:"));
            var kernelEntries = [];
            for (var _a = 0, _b = entry.kernels; _a < _b.length; _a++) {
                var kernel = _b[_a];
                kernelEntries.push(React.createElement("li", null,
                    React.createElement("code", null, kernel.display_name)));
            }
            entries.push((React.createElement("ul", null, kernelEntries)));
        }
    }
    var body = (React.createElement("div", null, entries));
    var prompt;
    if (kernelCompanions && serverCompanion) {
        prompt = 'Install Companions';
    }
    else if (kernelCompanions) {
        prompt = 'Install in Kernel';
    }
    else {
        prompt = 'Install Server Ext';
    }
    return apputils_1.showDialog({
        title: 'Kernel companions',
        body: body,
        buttons: [
            apputils_1.Dialog.cancelButton(),
            apputils_1.Dialog.warnButton({
                label: prompt,
                caption: 'Try to install the package into the selected kernels.'
            }),
            apputils_1.Dialog.okButton({
                label: 'Install Ext Only',
                caption: 'Install only the Jupyterlab frontend extension.'
            })
        ],
    }).then(function (result) {
        if (result.button.label === prompt) {
            return promptInstallCompanions(kernelCompanions, serverCompanion, serviceManager);
        }
        return result.button.label === 'Install Ext Only';
    });
}
exports.presentCompanions = presentCompanions;
/**
 * Prompt the user what do about companion packages, if present
 *
 * @param builder the build manager
 */
function promptInstallCompanions(kernelCompanions, serverCompanion, serviceManager) {
    // VDOM entries to put in dialog:
    var entries = [];
    // Config (model) to be filled by dialog:
    var config = {};
    var _loop_1 = function (entry) {
        var lookupName = entry.kernelInfo.base.name;
        var kernelEntries = [];
        // Create initial config:
        config[lookupName] = {
            kernelInfo: entry.kernelInfo,
            manager: entry.kernelInfo.managers[0] || '',
            selected: {},
        };
        var _loop_2 = function (kernel) {
            // For each entry, create a checkbox:
            kernelEntries.push((React.createElement("label", null,
                React.createElement("input", { type: "checkbox", value: kernel.name, onChange: function () {
                        // Have checkbox toggle modify config:
                        var selected = config[lookupName].selected;
                        if (kernel.name in selected) {
                            delete selected[kernel.name];
                        }
                        else {
                            selected[kernel.name] = kernel;
                        }
                    } }),
                kernel.display_name)), React.createElement("br", null));
        };
        for (var _i = 0, _a = entry.kernels; _i < _a.length; _i++) {
            var kernel = _a[_i];
            _loop_2(kernel);
        }
        // Add select for picking which package panager to use
        var managerOptions = [];
        for (var _b = 0, _c = entry.kernelInfo.managers || []; _b < _c.length; _b++) {
            var m = _c[_b];
            managerOptions.push(React.createElement("option", { value: m }, m));
        }
        entries.push(React.createElement("div", null,
            entry.kernelInfo.base.name,
            React.createElement("select", { onChange: function (event) {
                    config[lookupName].manager = event.target.value;
                } }, managerOptions),
            kernelEntries));
    };
    for (var _i = 0, kernelCompanions_2 = kernelCompanions; _i < kernelCompanions_2.length; _i++) {
        var entry = kernelCompanions_2[_i];
        _loop_1(entry);
    }
    var serverEntries = [];
    var serverManager = '';
    if (serverCompanion) {
        // Add select for picking which package manager to use
        var managerOptions = [];
        serverManager = serverCompanion.managers[0] || '';
        for (var _a = 0, _b = serverCompanion.managers || []; _a < _b.length; _a++) {
            var m = _b[_a];
            managerOptions.push(React.createElement("option", { value: m }, m));
        }
        managerOptions.push(React.createElement("option", { value: "-- Do nothing --" }));
        serverEntries.push(React.createElement("p", null,
            "Server extension install",
            React.createElement("code", null, serverCompanion.base.name),
            React.createElement("select", { onChange: function (event) {
                    serverManager = event.target.value;
                } }, managerOptions)));
    }
    var body = (React.createElement("div", null,
        serverEntries,
        React.createElement("p", null, "Which kernel(s) do you want to install into?"),
        entries));
    var dialogPromise = apputils_1.showDialog({
        title: 'Install kernel companions',
        body: body,
        buttons: [
            apputils_1.Dialog.cancelButton(),
            apputils_1.Dialog.warnButton({ label: 'Install' })
        ],
    });
    var installPromise = dialogPromise.then(function (result) {
        if (!result.button.accept) {
            return;
        }
        var _loop_3 = function (key) {
            var c = config[key];
            for (var _i = 0, _a = Object.keys(c.selected); _i < _a.length; _i++) {
                var kernelName = _a[_i];
                var spec = c.selected[kernelName];
                serviceManager.sessions.startNew({
                    path: coreutils_1.uuid(16),
                    kernelName: spec.name,
                }).then(function (session) {
                    var kernel = session.kernel;
                    var override = {};
                    if (c.kernelInfo.overrides && c.manager in c.kernelInfo.overrides) {
                        override = c.kernelInfo.overrides[c.manager];
                    }
                    var info = __assign({}, c.kernelInfo.base, override);
                    installOnKernel(kernel, c.manager, info).then(function () {
                        session.shutdown();
                    });
                });
            }
        };
        // Start launching kernels, build commands, and send commands
        for (var _i = 0, _a = Object.keys(config); _i < _a.length; _i++) {
            var key = _a[_i];
            _loop_3(key);
        }
        if (serverCompanion) {
            serviceManager.terminals.startNew().then(function (terminal) {
                var override = {};
                if (serverCompanion.overrides && serverManager in serverCompanion.overrides) {
                    override = serverCompanion.overrides[serverManager];
                }
                var info = __assign({}, serverCompanion.base, override);
                installOnServer(terminal, serverManager, info).catch(function () {
                    terminal.shutdown();
                });
            });
        }
    });
    return installPromise.then(function () {
        return dialogPromise;
    }).then(function (result) {
        if (!result.button.accept) {
            return false;
        }
        // Loop over all configured actions, and see if any bundle JS
        var bundled = false;
        for (var _i = 0, _a = Object.keys(config); _i < _a.length; _i++) {
            var key = _a[_i];
            var c = config[key];
            var override = {};
            if (c.kernelInfo.overrides && c.manager in c.kernelInfo.overrides) {
                override = c.kernelInfo.overrides[c.manager];
            }
            var info = __assign({}, c.kernelInfo.base, override);
            bundled = bundled || !!info.bundles_extension;
        }
        // If JS is bundled, prevent direct install of NPM package
        return !bundled;
    });
}
exports.promptInstallCompanions = promptInstallCompanions;
function installOnKernel(kernel, manager, info) {
    var code;
    if (manager === 'pip') {
        code = "\nimport sys\nfrom subprocess import check_call\ncheck_call([sys.executable, '-m', 'pip', 'install', '" + info.name + "'])\n";
    }
    else if (manager === 'conda') {
        code = "\nimport sys\nfrom subprocess import check_call\nimport os\npjoin = os.path.join\ncmd_opt = ['install', '--prefix', sys.prefix, '--yes', '--quiet', '" + info.name + "']\ntry:\n    check_call([pjoin(sys.prefix, 'bin', 'conda')] + cmd_opt)\nexcept FileNotFoundError:\n    if os.name == 'nt':\n        check_call([pjoin(sys.prefix, 'Scripts', 'conda')] + cmd_opt)\n    else:\n        raise\n";
    }
    if (code) {
        var future = kernel.requestExecute({
            code: code,
            stop_on_error: true
        });
        return future.done.then(function (reply) {
            if (reply.content.status !== 'ok') {
                console.error('Error installing on kernel', reply);
                throw new Error("Error installing on kernel:\n" + reply.content.status);
            }
        });
    }
    return Promise.reject("Unknown manager: " + manager);
}
function installOnServer(terminal, manager, info) {
    var cmd = '';
    if (manager === 'pip') {
        cmd += "pip install " + info.name;
    }
    else if (manager === 'conda') {
        cmd += "conda install --yes --quiet " + info.name;
    }
    cmd += '\r';
    return new Promise(function (resolve) {
        //let output = '';
        var onMessage = function (session, msg) {
            switch (msg.type) {
                case 'stdout':
                    if (msg.content) {
                        //output += msg.content[0];
                    }
                    break;
                case 'disconnect':
                    session.messageReceived.disconnect(onMessage);
                    // Process output?
                    resolve();
                    break;
                default:
                    break;
            }
        };
        terminal.ready.then(function () {
            terminal.messageReceived.connect(onMessage);
            terminal.send({
                type: 'stdin',
                content: [cmd]
            });
            terminal.send({
                type: 'stdin',
                content: ['exit\r']
            });
        });
    });
}
//# sourceMappingURL=companions.js.map